// Model.h

// TDL4 built May 29 2018, 18:09:31 
#define NFIXEF 5 // number of user fixef vars
#define NERROR 1 // number of user error vars
#define NRANEF1 0
#define NRANEF2 0
#define NRANEF3 0
#define NRANEF4 0
#define NRANEF5 2
#define MULTI_LEVEL_RANEF 0
#define NSECONDARY 0 // number of secondary parameters
#define TRULYTIMEBASED 1
#define ALLOWGAUSSIANFIT 1
#define ALLOWLOGTRANSFORM 0
#define MODELUNIQUEID 52

#define NDERIV 0 // number of user deriv vars
#define NURINE 0 // number of urine compartments
#define NTRANSIT2 0 // number of transit2 compartments
#define NTRANSIT 0 // number of transit compartments
#define NEVENT 0 // number of event/count statements
#define NINTEG_ALL (NDERIV + NURINE + NTRANSIT2 + NTRANSIT + NEVENT)
#define NCFMACRO 1 // number of closed form machines

#ifdef IN_MODEL
	#define DEFARRAY(typ, nam, siz) typ nam[siz]
	#define EXTERN
	ODECallbackTable* _pocb = 0;
	CallbackTable* _pcb = 0;
#else
	#define DEFARRAY(typ, nam, siz) extern typ nam[]
	#define EXTERN extern
	extern ODECallbackTable* _pocb;
	extern CallbackTable* _pcb;
#endif

DEFARRAY(double, zzY, NINTEG_ALL + 1 + 100); // integrator vector + TODO: space for gradients
DEFARRAY(double, zzIRate, NINTEG_ALL + 1);	// infusion drip rate
DEFARRAY(double, zzR, NINTEG_ALL + 1 + 100); // dummy rate array

DEFARRAY(double, _jacobian, (NINTEG_ALL+1) * (NINTEG_ALL+1));	// for matrix exponent
#undef EXTERN
#undef DEFARRAY

// reset drug compartments and infusion state to zero (at start of subject)
void ResetInteg();

// reset urine compartments to zero (at start of urine collection interval) 
void ResetUrine();

// reset event hazard accumulators to zero (at start of subject) 
void TDL4ResetEvent();

// reset gradient accumulators to zero (at start of subject) 
void ResetGradient();

// clear all integrators (at start of subject)
void InitSubject();

// restart subject sequence state machines
void RestartSubjectSequences();
void RestartSubjectEventSims();

// do miscellaneous actions at end of subject
void EndSubject(int _bSimulating);

// recompute all elements of the jacobian matrix whenever covariates change
void SetJacobian();

// set value of named covariate
void SetCovariate(double _t, const char* _nm, double _v);
// given current parameters and state, compute various variables and check for in bounds
void EvalGroup(double* zzY);
void EvalSParms(double* zzY);
void EvalAssign(double* zzY);

// set model time variable, if it is a time-based model
void SetModelTime(double _t);

// given fixed effects, compute secondary parameters
void CalcSecondary();

// advance the state of the accumulators for given interval of time
void Advance(int bGenGradients, int _iODELevel, double *_pTimeOuter, double _dt, int _bTimeAdvances);

// retrieve/set values of CF machines
void GetAllCFState(double *_y);
void SetAllCFState(double *_y);

// give a bolus to the indicated compartment
void Bolus1(double _t, int _iCpt, double _v);
void Bolus2(double _t, int _iCpt, double _v);

// start an infusion to the indicated compartment
void StartIV1(double _t, int _iCpt, double _amt, double _rate);
void StartIV2(double _t, int _iCpt, double _amt, double _rate);

void GetLL(int bGenGradients, double _dLL[], double _dGrad[], int *_pnObs, double _t, const char* _nm, double _amt, int _bBql);
void NewGetLL(int bGenGradients, double _dLL[], double _dGrad[], int *_pnObs, double _t, const char* _nm, double _amt, int _bBql);
void ClearSSIrrelevantState(double* _y);
void Deriv(int* _pN, double* _pt, double zzY[], double zzR[]);
void AddIncrements(double zzY[], double ta, double tb);
void GradientDeriv(int* _pN, double* _pt, double zzY[], double zzR[]);
void GetPred(int *_pnObs, double _t, const char* _nm, double _amt);
void Simulate(double _t, const char* _nm, double _v1, int _bBql, int bSampleEpsilon, int bForVPC);
void GetNumCategories(const char* _nm, int* pNCat);
void NewGetPred(int bGenGradients, int* pnObs, double _t, const char* _nm, double _v1);

double unif();
double norm();

